#include "wnd.h"
#include "ui_wnd.h"
#include <QtNetwork>
#include <QtGui>

// Debugging
#include<QtDebug>

wnd::wnd(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::wndClass)
{
    // Setuping User Interface from wnd.ui
    ui->setupUi(this);

    // Initializing Google Translate Engine
    googTr = new GoogleProto();
    connect( googTr, SIGNAL(translationComplete(QString)), this, SLOT(translationComplete(QString)) );

    // Setting up languages list
    QStringList
            translates = googTr->translatesFull ,
            list = googTr->translatesShort ;

    langs = new QStringList(list);

    // Connecting languages list to language selectors (ComboBoxes)
    ui->langFrom->addItems(translates);
    ui->langTo->addItems(translates);
    qDebug("Langs setted");

    // Setting up icons for ComboBoxes
    for(int i=0;i<ui->langFrom->count();i++)
    {
        QString path = QString(":/flags/res/flags/%1.png").arg(langs->at(i));
        ui->langFrom->setItemIcon(i,QIcon(path));
        ui->langTo->setItemIcon(i,QIcon(path));
    }
    qDebug("Lang flags configured...");

    // Setting up last used languages
    // If application runs firs time on this computer by default setting up English2English
    ui->langFrom->setCurrentIndex(settings.value("lang/from",10).value<int>());
    ui->langTo->setCurrentIndex(settings.value("lang/to",10).value<int>());
    qDebug("Last pair loaded");

    // Translated text must be readOnly
    ui->textTo->setReadOnly(true);

    // Setuping actions
    setUpActions();

    // Creating tray icon
    createTrayIcon();
    trayIcon->show();

    //Windows
    aboutAppWnd = new AboutAppWnd();
    settingsWnd = new SettingsWnd();

    //Debug output
    qDebug("All buttons are connected");
}
wnd::~wnd()
{
    // Saves current language pair
    settings.setValue("lang/from",ui->langFrom->currentIndex());
    settings.setValue("lang/to",ui->langTo->currentIndex());
    qDebug("Last language pair saved");

    qDebug("Good bye.");
    delete ui;
}
void wnd::closeEvent(QCloseEvent *event)
{
    if (trayIcon->isVisible()) {
        hide();
        event->ignore();
    }
}
void wnd::createTrayIcon()
{
    trayIconMenu = new QMenu(this);
    trayIconMenu->addAction( restoreAction );
    trayIconMenu->addSeparator();
    trayIconMenu->addAction( settingsAction );
    trayIconMenu->addSeparator();
    trayIconMenu->addAction( aboutQtAction );
    trayIconMenu->addAction( aboutAppAction );
    trayIconMenu->addSeparator();
    trayIconMenu->addAction( quitAction );

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setIcon( QIcon(":/res/icon.png") );
    trayIcon->setContextMenu(trayIconMenu);

    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}
void wnd::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
        case QSystemTrayIcon::MiddleClick:
        //case QSystemTrayIcon::Trigger:
        case QSystemTrayIcon::DoubleClick:
            show();
            break;
        default: break;
    }
}
void wnd::aboutQt() {
    QMessageBox::aboutQt(this);
}
void wnd::aboutApp() {
    aboutAppWnd->show();
}
void wnd::settingsWndOpen() {
    settingsWnd->show();
}
void wnd::setUpActions()
{
    cmdEnter = new QAction(tr("&Translate"), this);
    cmdEnter->setShortcut( Qt::SHIFT + Qt::Key_Return );
    ui->textFrom->addAction(cmdEnter);
    connect(cmdEnter, SIGNAL(triggered()), this, SLOT(on_translate_clicked()));

    restoreAction = new QAction(tr("&Restore"), this);
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

    quitAction = new QAction(tr("&Quit"), this);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

    aboutQtAction = new QAction(tr("Abou&t Qt"), this);
    connect( aboutQtAction, SIGNAL(triggered()), this, SLOT(aboutQt()) );

    aboutAppAction = new QAction(tr("&About App"), this);
    connect( aboutAppAction, SIGNAL(triggered()), this, SLOT(aboutApp()) );

    settingsAction = new QAction(tr("&Settings"), this);
    connect( settingsAction, SIGNAL(triggered()), this, SLOT(settingsWndOpen()) );
}

void wnd::on_swap_clicked()
{
    // Swapping languages (e.g. English<->Russian)
    int tmp = ui->langTo->currentIndex();
    ui->langTo->setCurrentIndex( ui->langFrom->currentIndex() );
    ui->langFrom->setCurrentIndex( tmp );
}
void wnd::on_translate_clicked()
{
    qDebug("Starting translation.");
    QString text;
    googTr->startTranslation( ui->langFrom->currentIndex(), ui->langTo->currentIndex(), ui->textFrom->toPlainText() );
}
void wnd::translationComplete( QString text)
{
    ui->textTo->setText( text );
}
